//3456789_123456789_123456789_123456789_123456789_123456789_123456789_123456789_
// High Contrast image viewer using LogLUV file format
//  
//  J. Tumblin 12/5/97
//
//  Brief TIFF summary: 
//    All TIFF files begin with 8-byte Image File Header (IFH), and 
//    after this contain
//    -- 1 or more Image File Directories (IFD), possibly interspersed with
//    -- chunks of data whose format and location are defined by the
//       contents of the IFDs.
//    IFH : Image File Header contains
//      -byte 0-1: word ordering (big-/little-endian) hex4949== LSb to MSb,
//                                                   hex4D4D== MSb to LSb.
//      -byte 2-3: 42 (hex 002A) the never-changing version #.
//      -byte 4-7  (long word). offset in bytes to the 1st IFDirectory,
//                measured from the 1st byte in file.  If IFD began at 1st 
//                byte, offset=0.
//    IFD's : Image File Directories contain:
//      -byte 0-1: 16 bit; count of # of 'directory entries' contained in this
//                directory, 
//      -followed by the each of the directory entries, stored sequentially 
//                in the directory in order of ascending 'Tag' number 
//                (which is found in the first part of each directory entry).
//      -concluding with 32-bit 'offset' to the 1st byte in the next Image
//                File Directory.  As before, 'offset' is measured in bytes
//                from the start of the file so that the first byte in file
//                has an offset of zero).
//    Directory Entries are 12-byte blocks that describe a chunk of data in
//                the file called a 'field'.
//           -byte 0-1:  'Tag'; a unique number that defines the use and meaning
//                        of the data the field holds, e.g. 'ImageWidth',
//                        'BitsPerSample', etc. Tags are explicitly defined 
//                        in the TIFF standard.
//           -byte 2-3:  'Field type' the data object type used in the 'field'.
//                        Allowable data types for each legal 'Tag' are also
//                        part of the TIFF standard; they include byte, 
//                        rational(fraction), 16-bit word, 32-bit word, etc.,
//           -byte 4-7:  'Field length'; the number of data objects stored in
//                        the field. (bytes = Field length*sizeof(FieldType))
//           -byte 8-11; 'Value Offset'; offset in bytes from the start-of-file
//                        to the first byte of the field; MUST be word-aligned,
//                        hence always even #d.  
//
// man cvd, use -g (debug info) and -v (verbose) for compile
//

#include "logluv.h"
extern int lastcall(char *str);


int main(int argc,char* argv[])
  //=========================================================================
{
  int i,j,tmp;
  uint16 xsiz,ysiz;
  HcFile inLuv;
  float Xavg,Yavg,Zavg,Xtmp,Ytmp,Ztmp;
  FILE *pFile;

  inLuv.putColorSys(hcCIE_XYZ);
  if(argv[1]==NULL) 
      {              //Open a logluv tiff file:---------------
      inLuv.readOpen("Images/barebulb.tif");
      } 
  else 
      {
      inLuv.readOpen(argv[1]); 	//  open 1st 'directory' in .tif file 
      }
  Xavg=Yavg=Zavg=Xtmp=Ytmp=Ztmp=0.0;
  inLuv.putColorSys(hcCIE_XYZ);	// set up for reading CIE XYZ values.
  xsiz = (uint16)inLuv.getXsize();
  ysiz = (uint16)inLuv.getYsize();
  for(i=0; i<xsiz;i++) 
      {				// clear the scanline buffer,
      inLuv.scanline[i]=0.0;
      inLuv.scanline[i+1]=0.0;
      inLuv.scanline[i+2]=0.0;
      }
  pFile = fopen("barebulb.jt","wb"); // make an output file;
  tmp = fwrite(&xsiz,sizeof(uint16),1,pFile);	// write array size;
  tmp += fwrite(&ysiz,sizeof(uint16),1,pFile);
  if(tmp!=2)
      {
      printf("\n!MATLAB file write error!");
      }
  for(j=0; j<(int)ysiz; j++) 
      {				// then for each scanline in file,
      inLuv.readLine((long)j);
      Xtmp = Ytmp = Ztmp=0.0;
      if(j==0) 
	  {
	  printf("1st 4 Z values:%f,%f,%f,%f\n",
		 inLuv.scanline[0],inLuv.scanline[3],
		 inLuv.scanline[6],inLuv.scanline[9]);
	  }
      for(i=0; i<(int)xsiz; i++)
	  {
	  Xtmp += inLuv.scanline[3*i]   * inLuv.getStoNits();
	  Ytmp += inLuv.scanline[3*i+1] * inLuv.getStoNits();
	  Ztmp += inLuv.scanline[3*i+2] * inLuv.getStoNits();
fwrite(&(inLuv.scanline[3*i]),sizeof(float),3,pFile);
// write X,Y,Z to file;
	  }
      Xtmp = Xtmp * 1.0f/((float)inLuv.getXsize());
      Ytmp = Ytmp * 1.0f/((float)inLuv.getXsize());
      Ztmp = Ztmp * 1.0f/((float)inLuv.getXsize());
      Xavg += Xtmp;
      Yavg += Ytmp;
      Zavg += Ztmp;
      }
  Xavg = Xavg/((float)inLuv.getYsize());
  Yavg = Yavg/((float)inLuv.getYsize());
  Zavg = Zavg/((float)inLuv.getYsize());
  printf("Xavg=%f,Yavg=%f,Zavg=%f\n",Xavg,Yavg,Zavg);
  inLuv.close();
  fclose(pFile);
  
  /*---------------------OLD STUFF!
  TIFF *htif_in;            // Handle for TIFF image struct; 
  uint32  width,height,linesz;
  float *p_line;                      // scanline buffer
  uint16 flag,phot,comp;
  double stonits;
  uint32 j;
  int tmp;
 
  if(argv[1]==NULL) 
      {              //Open a logluv tiff file:---------------
      htif_in = TIFFOpen("Images/barebulb.tif","r");
      } 
  else 
      {
      htif_in = TIFFOpen(argv[1],"r"); //  open 1st 'directory' in .tif file 
      }
  if(!htif_in) 
      {                   // if input file doesn't exist,
      lastcall("\nInvalid filename; bye. \n");
      }
  // Read its size & set up for Float CIE XYZ readout------------------------ 

  // Tell the SGILOG codec we want Float CIE X,Y,Z output;
  tmp = TIFFSetField(htif_in,TIFFTAG_SGILOGDATAFMT,SGILOGDATAFMT_FLOAT);
  printf("setfield returned %d\n",tmp);

  width = height = linesz = 0;   // (defaults if TIFFGetField() fails).
  TIFFGetField(htif_in, TIFFTAG_IMAGEWIDTH, &width);
  TIFFGetField(htif_in, TIFFTAG_IMAGELENGTH, &height);
  TIFFGetField(htif_in, TIFFTAG_PLANARCONFIG, &flag);
  if(flag!=PLANARCONFIG_CONTIG) 
      {
      // The tag 'TIFFTAG_PLANARCONFIG' tells how a scanline of the image is
      // organized.  Each pixel has 3 'samples' X,Y,Z; if PLANARCONFIG_CONTIG,
      // samples are contiguous, e.g. X1,Y1,Z1,X2,Y2,Z2,..., but if the tag is
      // PLANARCONFIG_SEPARATE then you get all the X's, then the Y's,  Z's.
      // Ward/Larson's files suggest (but don't demand) the first case.
      lastcall("Can't read TIFFs with non-adjacent samples!");
      }
  if(!TIFFGetField(htif_in, TIFFTAG_COMPRESSION, &comp)) 
      {
      comp = COMPRESSION_NONE;	// If no compression field in the file,
      //			// set field to 'no compression'
      }
  if(!TIFFGetFieldDefaulted(htif_in, TIFFTAG_PHOTOMETRIC, &phot)) 
      {
      // if file gives no phot value and no default value exists, exit.
      lastcall("\n TIFF has unspecified photometric type");
      }
  printf("\n width=%d, height=%d, Planarconfig=%d phot=%d, comp=%d\n",
	 width,height,flag,phot,comp);

  switch(phot) {
  case PHOTOMETRIC_LOGLUV:
  case PHOTOMETRIC_LOGL:
      if(comp != COMPRESSION_SGILOG && comp != COMPRESSION_SGILOG24) 
	  {
	  lastcall("\n Only support SGILOG compressed LogLuv data\n");
	  }
      break;
  default:
      lastcall("\nTIFF has unsupported photometric type\n");
      break;
  }
  if(!TIFFGetField(htif_in, TIFFTAG_STONITS, &stonits))
      {
      stonits = 1.0;// default value if none in file; cd/m^2 value for L=1.0 
      }
 
  linesz = (uint32)TIFFScanlineSize(htif_in);  // find size (bytes) of one
  // scanline as returned from TIFFReadScanline
  //? BUG? returns 2400bytes for a 400width scanline; 3*2bytes/pixel?!?!
  //  Should be returning width_in_pixels *3samples_in_pixel*sizeof(float)
  p_line = (float*)_TIFFmalloc(linesz);   // scanline buffer
  printf("\n scanlinesize = %d bytes \n",linesz);
  if(p_line==NULL) 
      {
      lastcall("hicont: could not malloc linebuffer!");
      }

  //find mean log luminance-----------------------------------
  printf("width=%d,height=%d\n\n",width,height);

  for(j=0; j<height; j++) 
      {  // for each scanline,
      tmp = TIFFReadScanline(htif_in,p_line,j);
      //printf("tmp=%d\n",tmp);
      if(tmp <1) 
	  {
	  lastcall("TIFFReadScanline() error"); // grab 1 row of pixel sampls
	  }
      }
  TIFFClose(htif_in);
  */
}

