//3456789_123456789_123456789_123456789_123456789_123456789_123456789_123456789_
//
//  File Format Converter:  converts to DOS/Windows/Intel 'lay' file
//				     from UNIX 'jt' file
// 
//	(it swaps the bytes in words: one machine is little endian but
//					the other is big-endian)
//	J. Tumblin 8/12/98
// 
//	'jt' format:
//     binary storage; uint16 xsize, ysize, followed by
//     a raster scanned sequence of (xsize*ysize) pixels, where 
//	each pixel is a set of 3 float32 numbers in R,G,B order.
//
//	Usage: jtunix2DOS infile.jt [outfile.lay]
//
//	output filename defaults to 'out.lay'.
// 

#include <stdio.h>
#include <string.h>

#define BYTE unsigned char
#define uint16 unsigned short

int main(int argc,char* argv[])
//=============================================================================
{
  int tmp;
  long i,imax;
  BYTE bbuf[4];		// buffer for 4 bytes of input data.
  BYTE btmp;
  uint16 xsiz,ysiz;
  FILE *pIn,*pOut;

  // for(i=0; i<argc; i++) 
  //  {
  //  printf("argv[%d]=%s\n",i,argv[i]);
  //  }
  if(argc==2 && strcmp(argv[1],"test")==0)
     {				// make a simple test file:
     printf("\nMade 16-byte file testDOS.jt; bytes hold 1,2,3,4,1,2,3,...\n");
     pOut = fopen("testDOS.jt","wb");
     for(i=0;i<4;i++) bbuf[i] = (BYTE)(i+1);
     printf("\nbbuf=%d,%d,%d,%d\n",bbuf[0],bbuf[1],bbuf[2],bbuf[3]);
     fwrite(bbuf,sizeof(BYTE),4,pOut);
     fwrite(bbuf,sizeof(BYTE),4,pOut);
     fwrite(bbuf,sizeof(BYTE),4,pOut);
     fwrite(bbuf,sizeof(BYTE),4,pOut);
     fclose(pOut);
     return(0);
     }
  if(argc==1 || argv[1][0]=='?') 
      {
      printf("\n\nFile Format Converter:\n");
      printf("\tconverts to DOS/Windows/ 'jt' file from UNIX 'jt' file\n");
      printf("\t one machine is big-endian, the other little-endian, so\n");
	     printf("\t program swaps the bytes.\n");
      printf("'jt' file format:\n");
      printf("binary storage; uint16 xsize, ysize, followed by\n");
      printf("a raster scanned sequence of (xsize*ysize) pixels, where \n");
      printf("each pixel is a set of 3 float32 numbers in R,G,B order.\n\n");
      printf("Usage: jtUnix2DOS infile.jt [outfile.lay]\n\n");

      printf("OR: 'jtUnix2 test' generates a test file 'testDOS.jt' \n");
      printf("\t that stores byte values 1,2,3,4,1,2,3,4,...\n");
      printf("output filename defaults to 'out.lay'\n\n");
      return(1);
      }
  pIn = fopen(argv[1],"rb");	// open input file for reading;
  if(pIn==NULL)
      {
      printf("\n jtUnix2DOS: could not open infile %s!\n",argv[1]);
      return(1);
      }
  if(argc>2)			// open output file for writing;
      {
      pOut = fopen(argv[2],"wb");
      }
  else
      {
      pOut = fopen("out.lay","wb");
      }
  if(pOut==NULL)
      {
      printf("\n jtUnix2DOS: could not open outfile!\n");
      fclose(pIn);
      return(1);
      }
  fread(&bbuf,1,4,pIn);	// read input file size in pixels, 1 byte at a time.
  xsiz = 256*(uint16)bbuf[0] + (uint16)bbuf[1];
  ysiz = 256*(uint16)bbuf[2] + (uint16)bbuf[3];
  printf("\n xsiz=%d,ysiz=%d\n",xsiz,ysiz);
  if(xsiz>4096 || ysiz>4096)
      {
      printf("Hunh? xsize=%d,ysize=%d is too big!\n",xsiz,ysiz);
      fclose(pIn);
      fclose(pOut);
      return(1);
      }
  btmp = bbuf[0];		// swap bytes for image size;
  bbuf[0] = bbuf[1];
  bbuf[1] = btmp;
  btmp = bbuf[2];
  bbuf[2] = bbuf[3];
  bbuf[3] = btmp; 
  fwrite(&bbuf,1,4,pOut); // write back in byte-reversed order;
  //fwrite(&ysiz,sizeof(uint16),1,pOut);
  imax = (long)xsiz * (long)ysiz * 3;
  for(i=0; i<imax; i++)
      {				// for every float value of every pixel,
      tmp = fread(&bbuf,1,4,pIn);	// read the 4 bytes of the float;
      if(tmp!=4)
	  {
	  printf("file read error at line%d/%d,pixel%d/%d,sample%d/3\n",
		 i/(3*xsiz),ysiz,i%(3*xsiz)/3,xsiz,(i%(3*xsiz))%3);
	  fclose(pIn);
	  fclose(pOut);
	  return(1);
	  }
      fwrite(&(bbuf[3]),1,1,pOut);	// write bytes in reversed order;
      fwrite(&(bbuf[2]),1,1,pOut);
      fwrite(&(bbuf[1]),1,1,pOut);
      fwrite(&(bbuf[0]),1,1,pOut);
      }
  fclose(pIn);
  fclose(pOut);
  return(0);
}
 
