//3456789_123456789_123456789_123456789_123456789_123456789_123456789_123456789_
//
//  Floating-point image shrinker:  reduces image size by 0.5,0.5
//
//	J. Tumblin 12/13/97
// 
//	'jt' image file format:
//     binary storage; UINT16 xsize, ysize, followed by
//     a raster scanned sequence of (xsize*ysize) pixels, where 
//	each pixel is a set of 3 float32 numbers in R,G,B order.
//
//	Usage: jtshrink infile.jt [outfile.jt]
//
//	output filename defaults to 'out.jt'.

#include <stdio.h>
#include <string.h>
#include <malloc.h>

#define BYTE unsigned char		// machine-dependent defines;
#define UINT16 unsigned short
#define FLOAT32 float

#define HMAX 3				// # of color components per pixel.

int jtclip(int in, int mmax);		//(a lonely function prototype)

int main(int argc,char* argv[])
//=============================================================================
{
  long h,hh,i,j,imax,jmax;
  size_t read1,read2;			// 
  FLOAT32 *prvBuf,*curBuf,*nxtBuf;	// 3 input scanline buffers; 
  FLOAT32 *outBuf;			// 1 output scanline buffer.
  FLOAT32 *ptmp;			// (placeholder for buffer swapping)
  int NWprv,Nprv,NEprv;			// input indices for filtering
  int Wcur,Ccur,Ecur;
  int SWnxt,Snxt,SEnxt;
  UINT16 insizX,insizY;			// input, output file sizes
  UINT16 outsizX,outsizY;
  FILE *pIn,*pOut;			// input, output file I/O pointers,
  char *pnameIn,*pnameOut;		// input, output file names.
  char defaultOutName[] = "out.jt";      

  // parse command line;----------------------------------------------
  // set default values;
  pnameIn = NULL;
  pnameOut = defaultOutName;
  outsizX = outsizY = insizX = insizY = 0;
  switch(argc) 
      {
      case 3:	// (jtshrink called with 2 args; must be infile,outfile names;
	  pnameIn = argv[1];	// just point to the names,
	  pnameOut = argv[2];
	  break;
      case 2:	// (jtshrink called with 1 arg; must be infile name;
	  pnameIn = argv[1];
	  if(strcmp("?",pnameIn)!=0)	// (skip over break & print usage msg 
	      {				// if the filename string == "?")
	      break;
	      }
      case 1:	// (jtshrink called with no args or '?')
      default:
	  printf("\nUsage: jtshrink infile.jt [outfile.jt] \n");
	  printf(" floating-point image shrinker for 'jt' files,\nwhere");
	  printf("\n\tinfile,outfile= input,output file names");
	  printf("\toutfile defaults to 'out.jt'\n");
	  printf("\toutfile xsize,ysize = infile xsize/2, ysize/2\n");
	  return(0);
      }
  // open input file, check for error:
  pIn = fopen(pnameIn,"rb");
  if(pIn==NULL)
      {
      printf("\n!!jtshrink: Could not open infile %s!!\n",pnameIn);
      return(1);
      }
  // get/set/check the size of in,out files;
  fread(&insizX,sizeof(UINT16),1,pIn);
  fread(&insizY,sizeof(UINT16),1,pIn);
  outsizX = insizX/2;		
  outsizY = insizY/2;
  if(insizX > 4096 || insizY > 4096)
      {				// if input is wildly oversized, complain;
      printf("\njtshrink: infile %s too large:%d x %d!!use jtDOS2unix?!?\n",
	     pnameOut,insizX,insizY);
      fclose(pIn);
      return(1);
      }
  // open output file, check for error:
  pOut = fopen(pnameOut,"wb"); 
  if(pOut==NULL)
      {
      printf("\n!!jtshrink: Could not open outfile %s!!\n",pnameOut);
      fclose(pIn);
      return(1);
      }
  // write output file size;
  fwrite(&outsizX,sizeof(UINT16),1,pOut);
  fwrite(&outsizY,sizeof(UINT16),1,pOut);
  // make 3 scanline-sized input file buffers;
  prvBuf = (FLOAT32 *)malloc(insizX*HMAX*sizeof(FLOAT32));// 'previous' line,
  curBuf = (FLOAT32 *)malloc(insizX*HMAX*sizeof(FLOAT32));// 'current' line,
  nxtBuf = (FLOAT32 *)malloc(insizX*HMAX*sizeof(FLOAT32));// 'next' line.
  // and one scanline-sized output file buffer.
  outBuf = (FLOAT32 *)malloc(outsizX*HMAX*sizeof(FLOAT32));
  if(prvBuf==NULL || curBuf==NULL || nxtBuf==NULL || outBuf==NULL) 
      {
      printf("\n!!jtshrink could not malloc linebuffers for %s!!\n",pnameIn);
      fclose(pOut);
      fclose(pIn);
      return(1);
      }
  imax = outsizX;	// (make 'int' to avoid promotion in 'for' stmts.)
  jmax = outsizY;
  for(j=0; j<jmax; j++) // for each output scanline,
      {
      ptmp = prvBuf;	// rotate buffers, read in next 2 input scanlines;
      prvBuf = nxtBuf;	
      nxtBuf = ptmp; 
      read1 = fread(curBuf,sizeof(FLOAT32),insizX*HMAX,pIn);
      read2 = fread(nxtBuf,sizeof(FLOAT32),insizX*HMAX,pIn);
      if(read1!=insizX*HMAX || read2!=insizX*HMAX)
	  {
	  printf("infile read error?!? read1=%d, read2=%d\n");
	  }
      if(j==0)
	  {		// for the first output scanline,
	  memcpy(prvBuf,curBuf,sizeof(FLOAT32)*insizX*HMAX);
	  }		// duplicate 'current' buffer in the 'prev' buffer.
      if(j==jmax-1)
	  {		// for the last output scanline,
	  memcpy(nxtBuf,curBuf,sizeof(FLOAT32)*insizX*HMAX);
	  }		// duplicate 'current' buffer in the 'next' buffer.
      for(i=0; i<imax; i++) // for each pixel in output scanline,
	  {
	  // calc input linebuffer indices for output pixel position (i,j),
	  Ccur  = jtclip(2*i,insizX)*HMAX;   
	  // and input linebuffer indices for 8 neighboring pixels;
	  Nprv  = jtclip(2*i,insizX)*HMAX;	// North (in prvBuf)
	  Snxt  = jtclip(2*i,insizX)*HMAX;	// South (in nxtBuf)
	  Ecur  = jtclip(2*i +1,insizX)*HMAX;	// East	(in curBuf)
	  Wcur  = jtclip(2*i -1,insizX)*HMAX;	// West (in curBuf)
	  NEprv = jtclip(2*i +1,insizX)*HMAX;	// NorthEast (in prvBuf)
	  NWprv = jtclip(2*i -1,insizX)*HMAX;	// NorthWest
	  SEnxt = jtclip(2*i +1,insizX)*HMAX;	// SouthEast
	  SWnxt = jtclip(2*i -1,insizX)*HMAX;	// SouthWest

	  for(h=0; h<HMAX; h++) 
	      {	// for each color component of each output pixel,
	      hh = HMAX*i + h;		// (output linebuffer index;
	      outBuf[hh] = 
		  //			// decimation filter; 
		  //			//	1 2 1
		  //			//	2 4 2
		  //			//	1 2 1 *(1/16)
		  curBuf[Ccur+h]*4.0f +
		  (prvBuf[Nprv+h] + 
		   nxtBuf[Snxt+h] + 
		   curBuf[Ecur+h] + 
		   curBuf[Wcur+h])*2.0f +
		  (prvBuf[NEprv+h] + 
		   prvBuf[NWprv+h] +
		   nxtBuf[SEnxt+h] +
		   nxtBuf[SWnxt+h]); 
	      outBuf[hh] *= 1.0f/16.0f;
	      }
	  }
      fwrite(outBuf,sizeof(FLOAT32),HMAX*imax,pOut);	// write output.
      }
  free(prvBuf);		// release scanline buffer memory.
  free(curBuf);
  free(nxtBuf);
  fclose(pIn);
  fclose(pOut);
  return(0);
}
 

int jtclip(int in,int mmax)
//========================================================================
// 0 <= returned_value < mmax;
{
  if(in>=mmax)
      {
      return(mmax-1);
      }
  if(in<0)
      {
      return(0);
      }
  return(in);
}
