//3456789_123456789_123456789_123456789_123456789_123456789_123456789_123456789_
// High Contrast image class using
// LogLUV file format (proposed part of TIFF standard)
//  
//  J. Tumblin 12/5/97
//  J. Tumblin 08/20/2002:  messy quick changes to read Debevec's HDRShop
//							format images: LogLUV EXCEPT he doesn't set the
//							proper tags (so I commented out error checks),
//							AND he does no LUV->RGB color-space conversion.

#include "hcfile.h"

HcFile::HcFile(void)
//==============================================================================
// Constructor sets the initial color type; 
// typically encode==jCIEXYZ, jRGB, jRGB_NTSC, or LUV
{
	fname[0]= NULL;
	xsize = ysize = line_num = 0;
	stordNits = 1.0;
	color_sys = hcRGB;		// Set default color encoding we'll use in 
	//				// the public buffer 'scanline'.
	r1w2 = 0;			// file state is NONE (neither read or write)
	sampleCnt = 0;		// no samples per pixel (usually 3 or 1).
	tiffbuf = NULL;
	scanline = NULL;
}

HcFile::~HcFile(void)
//==============================================================================
// Destructor: discards scanline and tiffbuf buffers.
{
}

int HcFile::readOpen(char *filename)
//==============================================================================
// Open a TIFF file for reading; return 1 on error; 0 if successful.
{
//HDRSHOP hack	int itmp;
uint16 flag,phot,comp;	    	// PLANARCONFIG value, PHOTMETRY value
								// and COMPRESSION values of file.
uint32 bufsz;					// tiffbuf buffer size in bytes;
char msg[hcNAMESIZE+64];

	printf("HcFile::ReadOpen():using %s on file%s\n",TIFFGetVersion(),filename);
	if(r1w2!=0)
	{
		sprintf(msg,"HcFile::ReadOpen() called with r1w2=%d!",r1w2);
		return(lastcall(msg));
	}
	if(filename!=NULL)
	{
		strncpy(fname,filename,hcNAMESIZE-1);	// (limited string length)
		h_tiff = TIFFOpen(fname,"r");	// open file for read, get file handle
	}					// and read 1st 'TIFF directory' 
	//					// (holds groups of TIFF tags)
	if(h_tiff==NULL)
	{
		sprintf(msg,"HcFile::ReadOpen() can't open file %s!",fname);
		return(lastcall(msg));   		// invalid filename;
	}
	// Make sure file holds image in a legal TIFF SGI LogLuv format;
	// 1) set up codec for Float CIE_XYZ output to tiffbuf using pseudo-tag;
//HDRShop hack		itmp  = TIFFSetField(h_tiff,TIFFTAG_SGILOGDATAFMT,SGILOGDATAFMT_FLOAT);
//HDRShop hack	if(itmp!=1)
//HDRShop hack	{
//HDRShop hack		return( lastcall("HcFile::ReadOpen():err on LOGLUV pseudotag!"));
//HDRShop hack	}
	TIFFGetField(h_tiff, TIFFTAG_PLANARCONFIG, &flag);
	if(flag!=PLANARCONFIG_CONTIG) 
	{
	// The tag 'TIFFTAG_PLANARCONFIG' tells how a scanline of the image is
	// organized.  Each pixel has 3 'samples' X,Y,Z; if PLANARCONFIG_CONTIG,
	// samples are contiguous, e.g. X1,Y1,Z1,X2,Y2,Z2,..., but if the tag is
	// PLANARCONFIG_SEPARATE then you get all the X's, then the Y's,  Z's.
	// Ward/Larson's 'ra_tiff' files seem to  demand the first case.
		return(lastcall("HcFile::ReadOpen():non-adjacent samples!"));
	}
	if(!TIFFGetField(h_tiff, TIFFTAG_COMPRESSION, &comp)) 
	{
		comp = COMPRESSION_NONE;	// If no compression field in the file,
						//			// set field to 'no compression'
	}
	if(!TIFFGetFieldDefaulted(h_tiff, TIFFTAG_PHOTOMETRIC, &phot)) 
	{
	// if file gives no photometric value and no default value exists, exit.
		return(lastcall("HcFile::ReadOpen():TIFF LogLuv: no photometric type"));
	}
	switch(phot) 			// Find, verify photometric type;
	{
		case PHOTOMETRIC_LOGL:	// TIFF logLuv grayscale image;
		case PHOTOMETRIC_LOGLUV:	// or TIFF logLuv color image;
			if(comp != COMPRESSION_SGILOG && comp != COMPRESSION_SGILOG24) 
			{
			return(lastcall(
					"HcFile::ReadOpen():non-SGILOG compression type!"));
			}
			break;
		default:
//HDRShop Hack			return(lastcall(
//HDRShop Hack					"HcFile::ReadOpen():TIFF has unknown photometric type"));
printf("!?!? phot=%d",phot);
phot = PHOTOMETRIC_LOGLUV; // FORCE it for Debevec's HDRshop images
//HDRShop hack (above)

	}
	// 					// Read image size;
	xsize = ysize = line_num = 0;   	// (defaults if TIFFGetField() fails).
	TIFFGetField(h_tiff, TIFFTAG_IMAGEWIDTH, &xsize);
	TIFFGetField(h_tiff, TIFFTAG_IMAGELENGTH, &ysize);
	if(!TIFFGetFieldDefaulted(h_tiff, TIFFTAG_SAMPLESPERPIXEL,&sampleCnt)) 
    {
      //if file gives no # of samples per pixel & no default value exists, exit.
		return(lastcall("HcFile::ReadOpen(): no # of samples/pixel found!"));
	}
	if(sampleCnt != 1 && sampleCnt != 3)	
    {				// tell me if 
		printf("\n\n!!! sample size was %d; defaulted to 1!!\n\n",sampleCnt);
		sampleCnt = 1;
    }
	if(!TIFFGetFieldDefaulted(h_tiff, TIFFTAG_STONITS, &stordNits))
    {
		stordNits = 1.0;// default value if none in file; cd/m^2 value for L=1.0 
    }
 
	printf("\n x,y,zsize=%d,%d,%d;Planarconfig=%d;phot=%d;comp=%d;stoNits=%f\n",
							xsize,ysize,sampleCnt,flag,phot,comp,stordNits);
	// find size (bytes) of one  scanline as returned from TIFFReadScanline
	// (Should return width_in_pixels*samples_in_pixel*sizeof(float))
	bufsz = (uint32)TIFFScanlineSize(h_tiff);  
	tiffbuf = (float *)_TIFFmalloc(bufsz); 	// (use TIFF library mem mgmt.
	printf("\n scanlinesize = %d bytes \n",bufsz);
	if(bufsz==NULL) 
	{
		return(lastcall("HcFile::ReadOpen(): could not malloc tiffbuffer!"));
    }
	scanline = (float *)_TIFFmalloc(3*xsize*sizeof(float));
	if(scanline==NULL) 
	{
		return(lastcall("HcFile::readOpen(): could not malloc scanline buffer!"));
	}
	r1w2 = 1;	      	// set 'open for read' flag.
	return(lastcall(0));	// normal exit.
}

int HcFile::readLine(long linenum)
//==============================================================================
// Retrieve line 'linenum' from already open file; put results (converted
// to the proper color system) in publicly available 'scanline' buffer.
// Return 0 if successful, 1 on error.
// ****UH-OH. we can only read files SEQUENTIALLY in logluv format!)
{
int err;

	if(r1w2!=1 || h_tiff==NULL || tiffbuf==NULL || scanline==NULL)
	{
		return(lastcall("HcFile::readLine():must readOpen() file first!"));
	}
	line_num = linenum;		// record it,
	err = TIFFReadScanline(h_tiff,tiffbuf,(uint32)line_num);  // read the line,
	if(err!=1) printf("err; linenumber%d\n",line_num);
	fromTiffbuf();		// do color-space convert & copy to 'scanline'
	return(lastcall(0)); 
}


int HcFile::writeOpen(char *filename,long sizeX, long sizeY, double stoNits)
//==============================================================================
// Open a new TIFFlogluv file for writing. CAREFUL! will destroy entire
//  contents of any existing file of the same name in the same directory.
// Returns 0 if successful.
{
	return(lastcall(0));
}

int HcFile::writeLine(long linenum)
//==============================================================================
// Write one scanline of data from 'scanline' to the file; interprets
// 'scanline' values according to color_sys. Returns 0 if successful.
{
	return(lastcall(0));
}

int HcFile::close(void)       
//==============================================================================
// Close the file.  Note we don't discard buffers. Return 0 if successful.
{
	if(h_tiff)			// if we have a valid TIFF file handle,
	{
		TIFFClose(h_tiff);
	}
	r1w2 = 0;			// r/w mode==NONE,
	if(tiffbuf!=NULL) 
	{
		_TIFFfree(tiffbuf);	// de-allocate scanline buffer.
	}
	if(scanline!=NULL)
	{
		_TIFFfree(scanline);
	}
	return(lastcall(0));
}

void HcFile::toTiffbuf(void)
//==============================================================================
{
/*  
int i;
float X,Y,Z;

	switch(color_sys)
	{
		case hcCIE_XYZ:
		for(i=0; i<3*xsize; i++) // just copy the scanline buffer to TIFF buf;
		{
			tifbuf[i] = scanline[i];
		}
		break;
		case hcRGB:
		for(i=0; i<xsize;i++) 
		{
			X = scanline[i*3];
			Y = scanline[i*3+1];
			Z = scanline[i*3+2];
		}
	}      
*/
}

void HcFile::fromTiffbuf(void)
//==============================================================================
{
  int i;
  float r,g,b;

	switch(color_sys)
	{
		case hcCIE_XYZ:
			switch(sampleCnt)
			{
				case 1:			// only one TIFF value per pixel;
					for(i=0;i<xsize;i++) 	// copy to each.
					{
						r = g = b = tiffbuf[i];
						scanline[i*3]   = r;
						scanline[i*3+1] = g;
						scanline[i*3+2] = b;
					}
					break;
				case 3:
					for(i=0;i<3*xsize;i++)// copy the TIFF buffer contents.
					{
						scanline[i] = tiffbuf[i];
					}
					break;
				default:
					lastcall("HcFile:fromTiffbuf(): invalid sampleCnt!");
			}
			break;
		case hcRGB:		// use Ward/Larson's Luv->RGB convert:
			switch(sampleCnt)
			{
				case 1:		// one sample per pixel; must be luminance L:
					for(i=0;i<xsize;i++) 	// copy to each.
					{
						r = g = b = tiffbuf[i];
						scanline[i*3]   = r;
						scanline[i*3+1] = g;
						scanline[i*3+2] = b;
					}
					break;
				case 3:		// 3 samples /pixel; must be (float) L-u-v
/*					for(i=0;i<xsize;i++) 
					{
						r =	 2.690f*tiffbuf[i*3] + 
							-1.276f*tiffbuf[i*3+1] +
							-0.414f*tiffbuf[i*3+2];
						g =	-1.022f*tiffbuf[i*3] +
							 1.978f*tiffbuf[i*3+1] +
							 0.044f*tiffbuf[i*3+2];
						b =	 0.061f*tiffbuf[i*3] +
							-0.224f*tiffbuf[i*3+1] +
							 1.163f*tiffbuf[i*3+2];
						scanline[i*3]   = r;
						scanline[i*3+1] = g;
						scanline[i*3+2] = b;
					}
*/
					//=======================HDRSHOP: violates LogLUV std======
					// and just uses RGB values (undocumented too!)
					for(i=0;i<xsize;i++) 
					{
						r =	 tiffbuf[i*3  ];
						g =	 tiffbuf[i*3+1];
						b =	 tiffbuf[i*3+2];
						scanline[i*3]   = r;
						scanline[i*3+1] = g;
						scanline[i*3+2] = b;
					}//========================================================
					break;
				default:
					lastcall("HcFile:fromTiffbuf(): invalid sampleCnt!");
			}
			break;
      case hcCIE_LUV:
      case hcRGB_NTSC:
      default:
	      lastcall("HcFile::fromTiffbuf(): invalid color_sys!");
      }
}

int lastcall(char* str) 
/*============================================================================*/
// Centralized error reporting for function exits:
//  1) put breakpoints here to catch errors,
//  2) Comment out printf() line to suppress verbose diagnostic comments
{
  if(str==NULL) 	// called with '0' argument; this means
      {
      return(0);	// normal function exit without comment.
      }
  else			// called with nonzero argument? then either:
      {			// 1) A normal exit that includes a diagnostic 
      //		//    comment line that begins 'note:',
      if(0==strncmp(str,"note:",4))
	  {	     
	  printf("\nNormal Exit. %s\n",str);
	  return(0);	 
	  }
      else
	  {		// OR 2) An error exit with diagnostic comment:
      	  printf("\nError Exit. %s\n",str);
	  return(1);
	  }
      }
}

